package com.loafle.overflow.module.sensor.service;

import com.loafle.overflow.commons.model.PageParams;
import com.loafle.overflow.commons.utils.PageUtil;
import com.loafle.overflow.commons.utils.StringConvertor;
import com.loafle.overflow.crawler.config.*;
import com.loafle.overflow.module.domain.model.Domain;
import com.loafle.overflow.module.infra.model.*;
import com.loafle.overflow.module.infra.service.InfraService;
import com.loafle.overflow.module.meta.model.MetaSensorStatus;
import com.loafle.overflow.module.meta.service.MetaSensorItemKeyService;
import com.loafle.overflow.module.probe.model.Probe;
import com.loafle.overflow.module.probe.service.ProbeService;
import com.loafle.overflow.module.sensor.dao.SensorDAO;
import com.loafle.overflow.module.sensor.model.Sensor;
import com.loafle.overflow.module.sensor.model.SensorItem;
import com.loafle.overflow.module.target.model.Target;
import org.codehaus.jackson.map.ObjectMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by insanity on 17. 6. 28.
 */
@Service("SensorService")
public class SensorService  {

    @Autowired
    SensorDAO sensorDAO;

    @Autowired
    private ProbeService probeService;

    @Autowired
    private InfraService infraService;

    @Autowired
    private SensorItemService sensorItemService;

    @Autowired
    private MetaSensorItemKeyService metaSensorItemKeyService;

    private Map<Integer, String> mappingMap = null;

    public Sensor regist(Sensor sensor) {
        return this.sensorDAO.save(sensor);
    }

//    public List<Sensor> readAllByTarget(Target target, PageParams pageParams) {
//        return this.sensorDAO.findAllByTarget(target, PageUtil.getPageRequest(pageParams));
//    }

    public Page<Sensor> readAllByDomain(Domain domain, PageParams pageParams) {

        List<Probe> probeList = this.probeService.readAllByDomain(domain);

        List<Target> targetList = this.infraService.readAllTargetByProbeList(probeList);

        return this.sensorDAO.findAllByTargetList(targetList, PageUtil.getPageRequest(pageParams));
    }

    public Page<Sensor> readAllByInfra(long infraId, PageParams pageParams) {
        Infra dbInfra = this.infraService.read(infraId);

        if(dbInfra == null || dbInfra.getTarget() == null) return null;

        return this.sensorDAO.findAllByTarget(dbInfra.getTarget(), PageUtil.getPageRequest(pageParams));

    }

    public Sensor read(String id) {
        return this.sensorDAO.findOne(Long.valueOf(id));
    }

    public void remove(Sensor sensor) {
        this.sensorDAO.delete(sensor);
    }

    public Sensor start(Sensor sensor) {
        MetaSensorStatus status = new MetaSensorStatus((short)1);
        sensor.setStatus(status);
        return this.sensorDAO.save(sensor);
    }

    public Sensor stop(Sensor sensor) {
        MetaSensorStatus status = new MetaSensorStatus((short)2);
        sensor.setStatus(status);
        return this.sensorDAO.save(sensor);
    }

    @Transactional
    public Sensor registSensorConfig(Sensor sensor, List<SensorItem> sensorItemList) {

        this.sensorDAO.save(sensor);

        for(SensorItem sensorItem : sensorItemList) {
            sensorItem.setSensor(sensor);
        }

        this.sensorItemService.registAll(sensorItemList);

        return sensor;
    }

    public String generateSensorConfig(Sensor sensor) throws IOException {

//        Sensor dbSensor = this.sensorDAO.findOne(sensor.getId());
        PageParams pageParams = new PageParams();
        pageParams.setPageNo(0);
        pageParams.setCountPerPage(Integer.MAX_VALUE);
        pageParams.setSortCol("id");
        pageParams.setSortDirection("descending");
        Page<SensorItem> dbItemList = this.sensorItemService.readAllBySensor(sensor, pageParams);

        List<SensorItem> sensorItems = dbItemList.getContent();

        if(sensorItems.size() <= 0) {
            return "";
        }
        Sensor dbSensor = sensorItems.get(0).getSensor();

        Infra infra = this.infraService.readByTarget(dbSensor.getTarget());

//        InfraHost infraHost = this.getInfraHost(infra);

        com.loafle.overflow.module.infra.model.InfraService infraService = (com.loafle.overflow.module.infra.model.InfraService)infra;

        Config config = new Config();
        config.setId(String.valueOf(sensor.getId()));

        com.loafle.overflow.crawler.config.Target target = new com.loafle.overflow.crawler.config.Target();
        Connection connection = new Connection();
        connection.setIp(StringConvertor.intToIp(infraService.getHost().getIp()));
        connection.setPort(String.valueOf(infraService.getPort()));
        connection.setPortType(infraService.getPortType());
        connection.setSsl(infraService.isTlsType());

        target.setConnection(connection);

        Map<String, Object> auth = new HashMap<>();
        auth.put("url", "jdbc:mysql://"+StringConvertor.intToIp(infraService.getHost().getIp())+":"+String.valueOf(infraService.getPort()));
        auth.put("id", "docker"); // FIXME: Auth Info
        auth.put("pw", "qwer5795QWER"); // FIXME: Auth Info

        target.setAuth(auth);

        config.setTarget(target);

        Schedule schedule = new Schedule();
        schedule.setInterval("5"); // FIXME: Interval

        config.setSchedule(schedule);

        Crawler crawler = new Crawler();
        crawler.setName(dbSensor.getCrawler().getName());
        crawler.setContainer("java_proxy"); // FIXME: if

        config.setCrawler(crawler);

        if(this.mappingMap == null) {
            this.mappingMap = this.metaSensorItemKeyService.readAllByCrawler(dbSensor.getCrawler());
        }

        Item item = new Item();
        Keys keys = null;
        List<Keys> keysList = new ArrayList<>();
        for(SensorItem sItem : sensorItems) {
            keys = new Keys();
//            sItem.getItem().get
            keys.setMetric(sItem.getItem().getKey());
            keys.setKey(this.mappingMap.get(sItem.getItem().getId()));
            keysList.add(keys);
        }
        item.setKeys(keysList);

        QueryInfo queryInfo = new QueryInfo();
        queryInfo.setQuery("show status");

        item.setQueryInfo(queryInfo);

        MappingInfo mappingInfo = new MappingInfo();
        mappingInfo.setParseDirection("row");
        mappingInfo.setValueColumn("Value");

        List<String> keyColumns = new ArrayList<>();
        keyColumns.add("Variable_name");

        mappingInfo.setKeyColumns(keyColumns);

        item.setMappingInfo(mappingInfo);

        List<Item> itemList = new ArrayList<>();
        itemList.add(item);

        config.setItems(itemList);


        ObjectMapper objectMapper = new ObjectMapper();

        return objectMapper.writeValueAsString(config);
    }


    private InfraHost getInfraHost(Infra infra) {

        switch (infra.getInfraType().getId()) {
            case 1:
                InfraMachine infraMachine = (InfraMachine)infra;
                break;
            case 2:
                InfraHost infraHost = (InfraHost)infra;
                return infraHost;
            case 3:
                InfraOS infraOS = (InfraOS)infra;
                break;
            case 4:
                InfraOSApplication infraOSApplication = (InfraOSApplication)infra;
                break;
            case 5:
                InfraOSDaemon infraOSDaemon = (InfraOSDaemon)infra;
            case 6:
                InfraOSPort infraOSPort = (InfraOSPort)infra;
                break;
            case 7:
                com.loafle.overflow.module.infra.model.InfraService infraOSService = (com.loafle.overflow.module.infra.model.InfraService)infra;
                return infraOSService.getHost();
        }
        return null;
    }
}
